#include "tmdlib/TMDlib.h"
#include <fstream>
#include <iomanip>
#include <iostream>
#include <vector>
#include <string>
using namespace TMDlib;
extern std::string pdfpath;
int icount = 0;
TMD::TMD(void) {
    icount = icount + 1;
}

// Function to test all TMDlib APIs for a given PDF set and parameters
void testAllAPIs(TMD& tmd, const std::string& name, double x, double xbar, double kt, double mu, bool printDetails = true) {
    if (printDetails) {
        std::cout << "\n------------------------------------------------------" << std::endl;
        std::cout << "Testing APIs with: PDF set = " << name 
                  << ", x = " << x << ", xbar = " << xbar 
                  << ", kt = " << kt << ", mu = " << mu << std::endl;
        std::cout << "------------------------------------------------------" << std::endl;
    }
    
    try {
        // API 11: TMDpdf with different parameter sets
        double up, ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, top, tbar, gluon, photon, z0, wplus, wminus, higgs;
        tmd.TMDpdf(x, xbar, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, top, tbar, gluon, photon, z0, wplus, wminus, higgs);
        std::cout << "API 11 Success" << std::endl;
        std::cout << "  up=" << up << ", ubar=" << ubar
                  << ", dn=" << dn << ", dbar=" << dbar
                  << ", strange=" << strange << ", sbar=" << sbar 
                  << ", charm=" << charm << ", cbar=" << cbar
                  << ", bottom=" << bottom << ", bbar=" << bbar
                  << ", top=" << top << ", tbar=" << tbar
                  << ", gluon=" << gluon << ", photon=" << photon
                  << ", z0=" << z0 << ", wplus=" << wplus
                  << ", wminus=" << wminus << ", higgs=" << higgs << std::endl;
    } catch (const std::exception& e) {
        std::cout << "API 11 failed: " << e.what() << std::endl;
    }

    try {
        // API 9: TMDpdf with different parameter sets
        double up, ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon, photon, z0, wplus, wminus, higgs;
        tmd.TMDpdf(x, xbar, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon, photon, z0, wplus, wminus, higgs);
        std::cout << "API 9: Success" << std::endl;
        std::cout << "  up=" << up << ", ubar=" << ubar
                  << ", dn=" << dn << ", dbar=" << dbar
                  << ", strange=" << strange << ", sbar=" << sbar 
                  << ", charm=" << charm << ", cbar=" << cbar
                  << ", bottom=" << bottom << ", bbar=" << bbar
                  << ", gluon=" << gluon << ", photon=" << photon
                  << ", z0=" << z0 << ", wplus=" << wplus
                  << ", wminus=" << wminus << ", higgs=" << higgs << std::endl;
    } catch (const std::exception& e) {
        std::cout << "API 9 failed: " << e.what() << std::endl;
    }

    try {
        // API 7: TMDpdf with different parameter sets
        double up, ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon, photon;
        tmd.TMDpdf(x, xbar, kt, mu, up, ubar, dn, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon, photon);
        std::cout << "API 7: Success" << std::endl;
        std::cout << "  up=" << up << ", ubar=" << ubar
                  << ", dn=" << dn << ", dbar=" << dbar
                  << ", strange=" << strange << ", sbar=" << sbar 
                  << ", charm=" << charm << ", cbar=" << cbar
                  << ", bottom=" << bottom << ", bbar=" << bbar
                  << ", gluon=" << gluon << ", photon=" << photon << std::endl;
    } catch (const std::exception& e) {
        std::cout << "API 7 failed: " << e.what() << std::endl;
    }

    try {
        // API 5: TMDpdf with different parameter sets
        double uval, dval, sea, charm, bottom, gluon, photon;
        tmd.TMDpdf(x, xbar, kt, mu, uval, dval, sea, charm, bottom, gluon, photon);
        std::cout << "API 5: Success" << std::endl;
        std::cout << "  uval=" << uval << ", dval=" << dval 
                  << ", sea=" << sea << ", charm=" << charm 
                  << ", bottom=" << bottom << ", gluon=" << gluon 
                  << ", photon=" << photon << std::endl;
    } catch (const std::exception& e) {
        std::cout << "API 5 failed: " << e.what() << std::endl;
    }

    try {
        // API 3: TMDpdf with different parameter sets
        std::vector<double> xpq(14);
        tmd.TMDpdf(x, xbar, kt, mu, xpq);
        std::cout << "API 3 : Success" << std::endl;
        std::cout << "  tbar=" << xpq[0] << ", bbar=" << xpq[1] << ", cbar=" << xpq[2] 
                  << ", sbar=" << xpq[3] << ", ubar=" << xpq[4] << ", dbar=" << xpq[5] 
                  << ", gluon=" << xpq[6] << ", dval+dbar= " << xpq[7]
                  << ", uval+ubar=" << xpq[8] << ", strange="<< xpq[9] 
                  << ", charm=" << xpq[10] << ", bottom=" << xpq[11] 
                  << ", top=" << xpq[12] << ", photon=" << xpq[13] << std::endl;
    } catch (const std::exception& e) {
        std::cout << "API 3 failed: " << e.what() << std::endl;
    }

    try {

        // API 1: TMDpdf with different parameter sets
        std::vector<double> result = tmd.TMDpdf(x, xbar, kt, mu);
        std::cout << "API 1: Success" << std::endl;
        std::cout << "  tbar=" << result[0] << ", bbar=" << result[1] << ", cbar=" << result[2] 
                  << ", sbar=" << result[3] << ", ubar=" << result[4] << ", dbar=" << result[5] 
                  << ", gluon=" << result[6] << ", dval+dbar= " << result[7]
                  << ", uval+ubar=" << result[8] << ", strange="<< result[9] 
                  << ", charm=" << result[10] << ", bottom=" << result[11] 
                  << ", top=" << result[12] << ", photon=" << result[13] << std::endl;
    } catch (const std::exception& e) {
        std::cout << "API 1 failed: " << e.what() << std::endl;
    }
    
    std::cout << std::endl;
}

int main(int argc, char **argv) {
    std::vector<std::string> pdfSets = {
        "GBWlight",
        "PB-NLO-HERAI+II-2018-set2",
        "Blueml",
        "GBWlight",
        "SV19_nnlo"
    };

    std::string name;
    if (argc > 2) {
        pdfpath = argv[1];
        name = std::string(argv[2]);
        std::cout << "Using command line specified PDF set: " << name << std::endl;
    } else {
        name = "SV19_nnlo"; // Default PDF set
        std::cout << "Using default PDF set: " << name << std::endl;
    }

    TMD tmd;
    tmd.setVerbosity(1); // Reduce verbosity for cleaner output

    int irep = 0;
    
    try {
        std::cout << "Initializing PDF set: " << name << std::endl;
        tmd.TMDinit(name, irep);
    } catch (const std::exception& e) {
        std::cout << "Error initializing PDF set " << name << ": " << e.what() << std::endl;
        return 1;
    }

    // Get boundary values
    double xmin = tmd.TMDgetXmin();
    double xmax = tmd.TMDgetXmax();
    double q2min = tmd.TMDgetQ2min();
    double q2max = tmd.TMDgetQ2max();

    // Print PDF set information
    std::cout << "\n=====================================================" << std::endl;
    std::cout << "PDF Set: " << name << std::endl;
    std::cout << "TMDSet Description: " << tmd.TMDgetDesc() << std::endl;
    std::cout << "Order alphas = " << tmd.TMDgetOrderAlphaS() << std::endl;
    std::cout << "order PDF    = " << tmd.TMDgetOrderPDF() << std::endl;
    std::cout << "xmin         = " << xmin << std::endl;
    std::cout << "xmax         = " << xmax << std::endl;
    std::cout << "q2min        = " << q2min << std::endl;
    std::cout << "q2max        = " << q2max << std::endl;
    std::cout << "num flav     = " << tmd.TMDgetNf() << std::endl;
    std::cout << "number       = " << tmd.TMDnumberPDF(name) << std::endl;
    std::cout << "=====================================================" << std::endl;

    double x_below = xmin > 1e-7 ? xmin * 0.5 : 1e-8;
    double x_within = std::sqrt(xmin * xmax); 
    double x_above = xmax * 1.5;

    double mu_below = std::sqrt(q2min) * 0.5;
    double mu_within = (std::sqrt(q2max) + std::sqrt(q2min)) * 0.5; 
    double mu_above = std::sqrt(q2max) * 1.5;

    double xbar = 0.0;
    double kt = 1.5;

    std::cout << "\nTest values for " << name << ":" << std::endl;
    std::cout << "x_below = " << x_below << " (xmin = " << xmin << ")" << std::endl;
    std::cout << "x_within = " << x_within << " (geometric mean of xmin and xmax)" << std::endl;
    std::cout << "x_above = " << x_above << " (xmax = " << xmax << ")" << std::endl;
    std::cout << "mu_below = " << mu_below << " (sqrt(q2min) = " << std::sqrt(q2min) << ")" << std::endl;
    std::cout << "mu_within = " << mu_within << " (mu_within)" << std::endl;
    std::cout << "mu_above = " << mu_above << " (sqrt(q2max) = " << std::sqrt(q2max) << ")" << std::endl;
    std::cout << "kt = " << kt << " (fixed value)" << std::endl;
    std::cout << "xbar = " << xbar << " (fixed value)" << std::endl;

    // Test with x values and mu within valid range
    std::cout << "\n===== Testing with different x values (mu within range) =====" << std::endl;
    std::cout << "Testing with x below xmin: " << x_below << " mu: " << mu_within << std::endl;
    testAllAPIs(tmd, name, x_below, xbar, kt, mu_within, false);
    
    std::cout << "Testing with x within range: " << x_within <<  " mu: " << mu_within << std::endl;
    testAllAPIs(tmd, name, x_within, xbar, kt, mu_within);
    
    std::cout << "Testing with x above xmax: " << x_above  <<  " mu: " << mu_within << std::endl;
    testAllAPIs(tmd, name, x_above, xbar, kt, mu_within, false);

    // Test with mu values and x within valid range
    std::cout << "\n===== Testing with different mu values (x within range) =====" << std::endl;
    std::cout << "Testing with mu below qmin: " << mu_below << " x_within: " << x_within << std::endl;
    testAllAPIs(tmd, name, x_within, xbar, kt, mu_below, false);
    
    std::cout << "Testing with mu within range: " << mu_within << " x_within: " << x_within << std::endl;
    testAllAPIs(tmd, name, x_within, xbar, kt, mu_within, false);
    
    std::cout << "Testing with mu above qmax: " << mu_above << " x_within: " << x_within << std::endl;
    testAllAPIs(tmd, name, x_within, xbar, kt, mu_above, false);

    // If no command line argument, test all PDF sets with all boundary tests
    if (argc <= 2) {
        std::cout << "\n===== Testing all other PDF sets with all boundary tests =====" << std::endl;
        for (const auto& pdfSet : pdfSets) {
            if (pdfSet != name) { // Skip the already tested set
                try {
                    std::cout << "\nTrying to initialize PDF set: " << pdfSet << std::endl;
                    TMD tmdTest;
                    tmdTest.setVerbosity(0);
                    tmdTest.TMDinit(pdfSet, irep);
                    
                    // Get boundaries for this PDF set
                    double set_xmin = tmdTest.TMDgetXmin();
                    double set_xmax = tmdTest.TMDgetXmax();
                    double set_q2min = tmdTest.TMDgetQ2min();
                    double set_q2max = tmdTest.TMDgetQ2max();
                    
                    std::cout << "\n=====================================================" << std::endl;
                    std::cout << "PDF Set: " << pdfSet << std::endl;
                    std::cout << "xmin=" << set_xmin << ", xmax=" << set_xmax 
                              << ", q2min=" << set_q2min << ", q2max=" << set_q2max << std::endl;
                    
                    // Calculate test values for this PDF set
                    double set_x_below = set_xmin > 1e-7 ? set_xmin * 0.5 : 1e-8;
                    double set_x_within = std::sqrt(set_xmin * set_xmax);
                    double set_x_above = set_xmax * 1.5;
                    
                    double set_mu_below = std::sqrt(set_q2min) * 0.5;
                    double set_mu_within = (std::sqrt(q2max) + std::sqrt(q2min)) * 0.5;
                    double set_mu_above = std::sqrt(set_q2max) * 1.5;
                    
                    std::cout << "\nTest values for " << pdfSet << ":" << std::endl;
                    std::cout << "x_below = " << set_x_below << " (xmin = " << set_xmin << ")" << std::endl;
                    std::cout << "x_within = " << set_x_within << std::endl;
                    std::cout << "x_above = " << set_x_above << " (xmax = " << set_xmax << ")" << std::endl;
                    std::cout << "mu_below = " << set_mu_below << std::endl;
                    std::cout << "mu_within = " << set_mu_within << std::endl;
                    std::cout << "mu_above = " << set_mu_above << std::endl;
                    std::cout << "kt = " << kt << " (fixed value)" << std::endl;
                    std::cout << "xbar = " << xbar << " (fixed value)" << std::endl;
                    
                    // Test with x values and mu within valid range
                    std::cout << "\n----- Testing with different x values (mu within range) -----" << std::endl;
                    std::cout << "Testing with x below xmin: " << set_x_below <<  ", mu_within: " << mu_within << std::endl;
                    testAllAPIs(tmdTest, pdfSet, set_x_below, xbar, kt, set_mu_within, false);
                    
                    std::cout << "Testing with x within range: " << set_x_within << ", mu_within: " << mu_within << std::endl;
                    testAllAPIs(tmdTest, pdfSet, set_x_within, xbar, kt, set_mu_within, false);
                    
                    std::cout << "Testing with x above xmax: " << set_x_above << ", mu_within: " << mu_within <<  std::endl;
                    testAllAPIs(tmdTest, pdfSet, set_x_above, xbar, kt, set_mu_within, false);
                    
                    // Test with mu values and x within valid range
                    std::cout << "\n----- Testing with different mu values (x within range) -----" << std::endl;
                    std::cout << "Testing with mu below qmin: " << set_mu_below << ", x_within: " << x_within << std::endl;
                    testAllAPIs(tmdTest, pdfSet, set_x_within, xbar, kt, set_mu_below, false);
                    
                    std::cout << "Testing with mu within range: " << set_mu_within << ", x_within: " << x_within << std::endl;
                    testAllAPIs(tmdTest, pdfSet, set_x_within, xbar, kt, set_mu_within, false);
                    
                    std::cout << "Testing with mu above qmax: " << set_mu_above << ", x_within: " << x_within << std::endl;
                    testAllAPIs(tmdTest, pdfSet, set_x_within, xbar, kt, set_mu_above, false);
                } catch (const std::exception& e) {
                    std::cout << "Failed to initialize PDF set " << pdfSet << ": " << e.what() << std::endl;
                }
            }
        }
    }

    return 0;
}
