#include "tmdlib/TMDlib.h"
#include <iostream>
#include <sstream>
#include <string>
using namespace std;
using namespace TMDlib;

// Root includes
#include "TH1.h"
#include "TH2.h"
#include "TFile.h"
#include "TCanvas.h"
#include <TApplication.h>
#include <TStyle.h>
#include <TROOT.h>
#include "TLegend.h"
#include "TLatex.h"
TMD::TMD(void) {
//    cout << "TMD object is being created "  << endl;
}


int main(int argc,char **argv) {

    TApplication* gMyRootApp = new TApplication("My ROOT Application", &argc, argv);

    double kt,mu,x, xbar=0. ;

    int kf;

    double up, ubar, down, dbar, strange, sbar,  charm, cbar, bottom, bbar, gluon1, gluon2, gluon3, gluon4, photon;
//string name ;

    double mz = 91.18;


// set constants
    x = 0.01 ;  /* longitudinal momentum x */
    mu = 100. ;   /* evolution scale p */

    // x = 0.15 ; /* for PV17_grid_ff_Pip */
    // mu = 10 ; /* for PV17_grid_ff_Pip */


    cout << " ************************************************" << endl;
    cout << " *             TMDplotter (simple)              *" << endl;
    cout << " ************************************************" << endl;
    cout << " at x = " << x << " mu = " << mu << endl;



    int npoint=10 ;

// histogram declaration
    const int nbins=15;
    double ktval[nbins+1] = {0.1, 0.3, 0.7, 1., 5., 10., 20., 30., 40., 50., 80., 100., 200., 300., 400., 500. };
    TH1F *h_pdf1 = new TH1F("pdf set 1 at #mu=100 GeV vrs kt ","xpdf1",nbins,ktval );
    TH1F *h_pdf2 = new TH1F("pdf set 2 at #mu=100 GeV vrs kt ","xpdf2",nbins,ktval);
    TH1F *h_pdf3 = new TH1F("pdf set 3 at #mu=100 GeV vrs kt ","xpdf2",nbins,ktval);
    TH1F *h_pdf4 = new TH1F("pdf set 4 at #mu=100 GeV vrs kt ","xpdf2",nbins,ktval);

//


    //string name1="PB-NLO-HERAI+II-2018-set2";
    string name1="ccfm-setB0";
    string name2="PB-NLO-HERAI+II-2018-set1";
    string name3="KS-2013-linear";
    //string name4="PB-NLO_ptoPb208-set2";
    string name4="PV17_grid_pdf";

    TMD TMDtest1;
    TMD TMDtest2;
    TMD TMDtest3;
    TMD TMDtest4;
    int irep1=1, irep2=10, irep3=0,irep4=0 ;
    TMDtest1.TMDinit(name1,irep1) ;
    cout << " TMDSet Description: " << TMDtest1.TMDgetDesc() << endl;

    //cout << " after ini first set " << endl;
    TMDtest2.TMDinit(name2,irep2) ;
    cout << " TMDSet Description: " << TMDtest2.TMDgetDesc() << endl;
    //cout << " after ini 2nd set " << endl;
    TMDtest3.TMDinit(name3,irep3) ;
    cout << " TMDSet Description: " << TMDtest3.TMDgetDesc() << endl;
    //cout << " after ini 3nd set " << endl;
    TMDtest4.TMDinit(name4,irep4) ;
    cout << " TMDSet Description: " << TMDtest4.TMDgetDesc() << endl;
    //cout << " after ini 4th set " << endl;

    // cout<<" test1 " << TMDtest1.TMDgetXmin() <<  " test2 " << TMDtest2.TMDgetXmin() << " test3 " << TMDtest3.TMDgetXmin() << " test4 " << TMDtest4.TMDgetXmin() <<endl;
    // cout<<" test1 " << TMDtest1.TMDalphas(mz) <<  " test2 " << TMDtest2.TMDalphas(mz) <<" test3 " << TMDtest3.TMDalphas(mz) << " test4 " << TMDtest4.TMDalphas(mz) <<endl;


    for (int j=0; j<=nbins; j++) {
        kt = ktval[j];

        TMDtest1.TMDpdf(x,xbar,kt,mu, up, ubar, down, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon1, photon);
        h_pdf1 -> Fill(kt,gluon1) ;
//              h_pdf1 -> Fill(kt,up) ;
        // cout << " x " << x << " gluon1 = " << gluon1 <<  endl;

        TMDtest2.TMDpdf(x,xbar,kt,mu, up, ubar, down, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon2, photon);
        h_pdf2 -> Fill(kt,gluon2) ;
//              h_pdf2 -> Fill(kt,up) ;
        TMDtest3.TMDpdf(x,xbar,kt,mu, up, ubar, down, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon3, photon);
        h_pdf3 -> Fill(kt,gluon3) ;
//              h_pdf3 -> Fill(kt,up) ;
        TMDtest4.TMDpdf(x,xbar,kt,mu, up, ubar, down, dbar, strange, sbar, charm, cbar, bottom, bbar, gluon4, photon);
        h_pdf4 -> Fill(kt,gluon4) ;
//              h_pdf4 -> Fill(kt,up) ;

        //  cout << " x = " << x << " gluon1 = " << gluon1 << " gluon2 = " << gluon2 << " gluon3 = " << gluon3 << " gluon4 = "<< gluon4 <<  endl;

    }

//general root settings
    gROOT->Reset();
    gROOT->SetStyle("Plain");
    gStyle->SetPadTickY(1); // ticks at right side
    gStyle->SetOptStat(0); // get rid of statistics box
    TCanvas *c = new TCanvas("ctest", "",0, 0, 500, 500);
// divide the canvas in 1 parts in x and 1 in y

    c -> Divide(1,1);
    c -> cd(1);
    double col[50];
    col[1]=2;
    col[2]=4;
    col[3]=6;
    col[4]=8;
    h_pdf1 -> SetLineColor(col[1]);
    h_pdf2 -> SetLineColor(col[2]);
    h_pdf3 -> SetLineColor(col[3]);
    h_pdf4 -> SetLineColor(col[4]);
    gPad->SetLogy();
    gPad->SetLogx();
    double ymin1=0.001, ymax1=10.1;
    double ktmin=ktval[0];
    double ktmax=ktval[nbins];


    TH2F *frame1 = new TH2F("frame1","",100,ktmin,ktmax,10,ymin1,ymax1);

    frame1->SetTitle(";k_{t}; xA(x,#mu,k_{t})");
    frame1->Draw();

    TLatex *text = new TLatex(0.5,5,"#mu=100 GeV, x=0.01 ");
    text->SetTextSize(0.03);
    text -> Draw("same");


    TLegend* mylegend = new TLegend(0.4,0.7,0.5,0.8);
    mylegend->SetFillColor(0); // white background
    mylegend->SetBorderSize(0); // get rid of the borders
    mylegend->SetTextSize(0.03);
    const char *histo_name1 = name1.c_str();
    const char *histo_name2 = name2.c_str();
    const char *histo_name3 = name3.c_str();
    const char *histo_name4 = name4.c_str();
    mylegend->AddEntry(h_pdf1,histo_name1,"l");
    mylegend->AddEntry(h_pdf2,histo_name2,"l");
    mylegend->AddEntry(h_pdf3,histo_name3,"l");
    mylegend->AddEntry(h_pdf4,histo_name4,"l");

    mylegend->Draw();

    h_pdf1 -> Draw("histosame");
    h_pdf2 -> Draw("histosame");
    h_pdf3 -> Draw("histosame");
    h_pdf4 -> Draw("histosame");
    c-> Draw();
    c-> WaitPrimitive();
    gMyRootApp->SetReturnFromRun(true);

    return 0;
}
