*-- Author : S. Baranov
      DOUBLE PRECISION FUNCTION BLUEML(x,g,Q)
!---------------------------------------------------------------------------  
!> @author 
!> S. Baranov (LPI, Moscow)
!> @brief
!> calculates BFKL type unintegrated gluon, 
!> @brief
!> reference: hep-ph/9506446, DESY-95-125
!! @param[in]	x x-value
!! @param[in]	g kt2-value
!! @param[in]	Q Q2-value
!
!> BLUEML  x*gluon(x,kt2,Q2)
!
!---------------------------------------------------------------------------  
   
C*******************************************************************C
C     Gluon distribution by the method of J.Bluemlein               C
C      BLUEML = x*gluon(x,kt2,Q2)             DESY-95-125           C
C*******************************************************************C
      IMPLICIT DOUBLE PRECISION(A-G,P-Z)
      EXTERNAL BJ0,BI0
      COMMON/CONVOL/cx,cg2,cQ2
      COMMON/CONST/PI,ALP,ALS
c      write(6,*) ' in blueml start ',x,g2,q2
C... Parameters and constants
*! pi constant
      PI  = 3.1415926d0
*! electromagnetic constant
      ALP = 1.d0/137.d0
*! strong coupling constant
      g2 = g**2
      q2 = q**2
      ALS = 0.2d0
      BLUEML=0.D0
      cx = x
      cg2= g2
      cQ2= dabs(Q2)
*!Precision of integration
      acc= 2.D-3
      IF(cg2.GE.cQ2) THEN
         BLUEML=DGAUSS(BI0, x, 1.D0, acc)
      ENDIF
      IF(cg2.LT.cQ2) THEN
         BLUEML=DGAUSS(BJ0, x, 1.D0, acc)
      ENDIF
      BLUEML=DMAX1(0.D0,BLUEML/cg2)
c      write(6,*) ' in blueml ',x,g2,q2
      RETURN
      END

      DOUBLE PRECISION FUNCTION BJ0(eta)
      IMPLICIT DOUBLE PRECISION(A-G,P-Z)
      COMMON/CONST/PI,ALP,ALS
      COMMON/CONVOL/cx,cg2,cQ2
	Double Precision XGLU
	External XGLU
      Alsb= ALS*3.d0/PI
      argj= 2.d0*dsqrt(-Alsb*dlog(eta)*dabs(dlog(cg2/cQ2)))
      argg= cx/eta
      BJ0 = Alsb/eta * DBESJ0(argj) * XGLU(argg,cQ2)
      RETURN
      END

      DOUBLE PRECISION FUNCTION BI0(eta)
      IMPLICIT DOUBLE PRECISION(A-G,P-Z)
      COMMON/CONST/PI,ALP,ALS
      COMMON/CONVOL/cx,cg2,cQ2
	Double Precision XGLU
	External XGLU
      Alsb= ALS*3.d0/PI
      argj= 2.d0*dsqrt(-Alsb*dlog(eta)*dabs(dlog(cg2/cQ2)))
      argg= cx/eta
      BI0 = Alsb/eta * DBESI0(argj) * XGLU(argg,cQ2)
      RETURN
      END
* Kernlib
*
*
      Double Precision FUNCTION BESI0(RX)
      Double Precision RX
      CHARACTER*6 ENAME
      LOGICAL LEX
      DOUBLE PRECISION X,Y,R,V,V1,V4,V5,A0,A,A1,A2,B,B0,B1,B2,T(6)
      DOUBLE PRECISION W,W0,W1,W2,W3,W4,W5,W6
      DOUBLE PRECISION F,F1,F2,F3,C,C0,C1(0:15),PI1,EPS,H,ALFA,D
      DOUBLE PRECISION ZERO,ONE,TWO,FIVE,EIGHT
      DOUBLE PRECISION DBESI0,DEBSI0,DX
 
      DATA ZERO /0.0D0/, ONE /1.0D0/, TWO /2.0D0/, FIVE /5.0D0/
      DATA EIGHT /8.0D0/
      DATA PI1 /0.39894 22804 0143D0/, EPS /1.0D-14/
 
      DATA T(1) /0.66666 66666 6667D0/
      DATA T(2) /0.80000 00000 0000D0/
      DATA T(3) /0.07500 00000 0000D0/
      DATA T(4) /0.33333 33333 3333D0/
      DATA T(5) /0.20000 00000 0000D0/
      DATA T(6) /0.02500 00000 0000D0/
 
      DATA C1( 0) /+1.00827 92054 587D0/
      DATA C1( 1) /+0.00844 51226 249D0/
      DATA C1( 2) /+0.00017 27006 308D0/
      DATA C1( 3) /+0.00000 72475 911D0/
      DATA C1( 4) /+0.00000 05135 877D0/
      DATA C1( 5) /+0.00000 00568 170D0/
      DATA C1( 6) /+0.00000 00085 131D0/
      DATA C1( 7) /+0.00000 00012 384D0/
      DATA C1( 8) /+0.00000 00000 298D0/
      DATA C1( 9) /-0.00000 00000 790D0/
      DATA C1(10) /-0.00000 00000 331D0/
      DATA C1(11) /-0.00000 00000 045D0/
      DATA C1(12) /+0.00000 00000 018D0/
      DATA C1(13) /+0.00000 00000 010D0/
      DATA C1(14) /+0.00000 00000 000D0/
      DATA C1(15) /-0.00000 00000 001D0/
      ROUND(D)  =  SNGL(D+(D-DBLE(SNGL(D))))
 
      X=RX
      ENAME=' BESI0'
      LEX=.FALSE.
      GOTO 9
 
      ENTRY EBESI0(RX)
      X=RX
      ENAME='EBESI0'
      LEX=.TRUE.
      GOTO 9
      ENTRY DBESI0(DX)
      X=DX
      ENAME='DBESI0'
      LEX=.FALSE.
      GO TO 9
      ENTRY DEBSI0(DX)
      X=DX
      ENAME='DEBSI0'
      LEX=.TRUE.
 
    9 V=ABS(X)
      IF(V .LT. EIGHT) THEN
       Y=(V/TWO)**2
       W=Y**2
       A0=ONE
       A1=ONE+T(1)*Y
       A2=ONE+T(2)*Y+T(3)*W
       B0=ONE
       B1=ONE-T(4)*Y
       B2=ONE-T(5)*Y+T(6)*W
       W1=FIVE
       V1=ONE
       C=ZERO
       F=TWO
    3  C0=C
       F=F+ONE
       W1=W1+TWO
       W2=W1-ONE
       W3=W2-ONE
       W4=W3-ONE
       W5=W4-ONE
       W6=W5-ONE
       V1=V1+ONE
       V4=F*W4
       V5=Y/W5
       W=Y*V1/(V4*W3)
       W0=W*V5
       F1=ONE+Y*V1/(V4*W1)
       F2=W+F*W0/(V1*W2)
       F3=-W0*V5/(W4*W6)
       A=F1*A2+F2*A1+F3*A0
       B=F1*B2+F2*B1+F3*B0
       C=A/B
       IF(ABS((C0-C)/C) .GE. EPS) THEN
        A0=A1
        A1=A2
        A2=A
        B0=B1
        B1=B2
        B2=B
        GO TO 3
       ENDIF
       B1=C
       IF(LEX) B1=EXP(-V)*B1
      ELSE
       R=ONE/V
       Y=EIGHT*R
       H=TWO*Y-ONE
       ALFA=-TWO*H
       B1=ZERO
       B2=ZERO
       DO 1 I = 15,0,-1
       B0=C1(I)-ALFA*B1-B2
       B2=B1
       B1=B0
    1  CONTINUE
       B1=PI1*SQRT(R)*(B0-H*B2)
       IF(.NOT.LEX) B1=EXP(V)*B1
      ENDIF
      IF(LEX)  THEN
         IF(ENAME .EQ. 'EBESI0')  THEN
            EBESI0=ROUND(B1)
         ELSE
            DEBSI0=B1
         ENDIF
      ELSE
         IF(ENAME .EQ. ' BESI0')  THEN
            BESI0=ROUND(B1)
         ELSE
            DBESI0=B1
         ENDIF
      ENDIF
      RETURN
      END
* Kernlib
*
*
      DOUBLE PRECISION FUNCTION DGAUSS(F,A,B,EPS)
      DOUBLE PRECISION F,A,B,EPS
      DOUBLE PRECISION W(12),X(12),AA,BB,C1,C2,U,S8,S16,CONST
      LOGICAL MFLAG,RFLAG
      EXTERNAL F
C
C     ******************************************************************
C
C     ADAPTIVE DOUBLE PRECISION GAUSSIAN QUADRATURE.
C
C     DGAUSS IS SET EQUAL TO THE APPROXIMATE VALUE OF THE INTEGRAL OF
C     THE FUNCTION F OVER THE INTERVAL (A,B), WITH ACCURACY PARAMETER
C     EPS.
C
C     ******************************************************************
C
      DATA W / 0.10122 85362 90376 259D0,
     1         0.22238 10344 53374 471D0,
     2         0.31370 66458 77887 287D0,
     3         0.36268 37833 78361 983D0,
     4         0.27152 45941 17540 949D-1,
     5         0.62253 52393 86478 929D-1,
     6         0.95158 51168 24927 848D-1,
     7         0.12462 89712 55533 872D0,
     8         0.14959 59888 16576 732D0,
     9         0.16915 65193 95002 538D0,
     A         0.18260 34150 44923 589D0,
     B         0.18945 06104 55068 496D0/
 
      DATA X / 0.96028 98564 97536 232D0,
     1         0.79666 64774 13626 740D0,
     2         0.52553 24099 16328 986D0,
     3         0.18343 46424 95649 805D0,
     4         0.98940 09349 91649 933D0,
     5         0.94457 50230 73232 576D0,
     6         0.86563 12023 87831 744D0,
     7         0.75540 44083 55003 034D0,
     8         0.61787 62444 02643 748D0,
     9         0.45801 67776 57227 386D0,
     A         0.28160 35507 79258 913D0,
     B         0.95012 50983 76374 402D-1/
C
C     ******************************************************************
C
C  START.
      DGAUSS=0.0D0
      IF(B.EQ.A) RETURN
      CONST=0.005D0/(B-A)
      BB=A
C
C  COMPUTATIONAL LOOP.
    1 AA=BB
      BB=B
    2    C1=0.5D0*(BB+AA)
         C2=0.5D0*(BB-AA)
         S8=0.0D0
         DO 3 I=1,4
            U=C2*X(I)
            S8=S8+W(I)*(F(C1+U)+F(C1-U))
    3    CONTINUE
         S8=C2*S8
         S16=0.0D0
         DO 4 I=5,12
            U=C2*X(I)
            S16=S16+W(I)*(F(C1+U)+F(C1-U))
    4    CONTINUE
         S16=C2*S16
         IF( ABS(S16-S8) .LE. EPS*(1.+ABS(S16)) ) GO TO 5
         BB=C1
         IF( 1.D0+ABS(CONST*C2) .NE. 1.D0) GO TO 2
      DGAUSS=0.0D0
ccc      CALL KERMTR('D103.1',LGFILE,MFLAG,RFLAG)
      IF(MFLAG) THEN
         IF(LGFILE.EQ.0) THEN
            WRITE(*,6)
         ELSE
            WRITE(LGFILE,6)
         ENDIF
      ENDIF
      IF(.NOT. RFLAG) stop
      RETURN
    5 DGAUSS=DGAUSS+S16
      IF(BB.NE.B) GO TO 1
      RETURN
C
    6 FORMAT( 4X, 'FUNCTION DGAUSS ... TOO HIGH ACCURACY REQUIRED')
      END
      FUNCTION XGLU(XX,QQ2)
!---------------------------------------------------------------------------  
!> @brief
!> gluon density xg(x,Q2) for CT10 from LHAPDF
!! @param[in]
!> XX x-value
!! @param[in]
!> QQ2 Q2-value
! 
!---------------------------------------------------------------------------  
   
      Implicit none

      DOUBLE PRECISION XX,QQ2,XGLU,f(-6:6)
      DOUBLE PRECISION XXX,QQ
      LOGICAL PDFFIRST
      COMMON /W50516/PDFFIRST
      QQ=SQRT(MAX(0.D0,QQ2))
      XXX = XX


c call PDFSET each time
c r-kord: CT10 PDFset does not exists for blueml!
      call InitPDFsetByName("CT10")
c      call InitPDFsetByName("cteq6l1")
      call InitPDF(0)
      call evolvePDF(xxx,qq,f)
      
      XGLU = f(0)
      RETURN
      END
